/*
 * TOFU Circuit Simulator
 * Copyright (C) 2007 Michael Birken
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 * 
 * http://www.meatfighter.com/tofu
 */

package tofu.harryporter;

import tofu.*;

public class Bulb8 extends Component implements IBulb {

  private Node[] inputNodes = new Node[8];
  private Voltage[] lastVoltages = new Voltage[8];
  private String name;
  private String printString;
  
  public void init() {
    for(int i = 0; i < inputNodes.length; i++) {
      inputNodes[i] = getNode("i" + i);
    }
    name = getInitParam();
    Halter.addBulb(this);
  }

  public void update() {    
    boolean changed = false;
    StringBuilder sb = new StringBuilder();
    sb.append(name).append(": ");
    int value = 0;
    for(int i = 7; i >= 0; i--) {
      if (inputNodes[i].getVoltage() != lastVoltages[i]) {
        changed = true;        
      }
      lastVoltages[i] = inputNodes[i].getVoltage();
      value <<= 1;
      if (lastVoltages[i] == Voltage.H) {
        sb.append("1");
        value |= 1;
      } else {
        sb.append("0");
      }
    }
    if (changed) {      
      sb.append(" ");
      if (value < 0x10) {
        sb.append("0");
      }
      sb.append(Integer.toString(value, 16).toUpperCase());
      sb.append(" ").append(value).append(" ").append((byte)value);
      printString = sb.toString();
    }
  }

  public void flow() {
  }
  
  public void print() {
    System.out.println(printString);
  }
  
  public String getName() {
    return name;
  }

  public int compareTo(IBulb bulb) {
    return name.compareTo(bulb.getName());
  }  
}
