/*
 * TOFU Circuit Simulator
 * Copyright (C) 2007 Michael Birken
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 * 
 * http://www.meatfighter.com/tofu
 */

package tofu.harryporter;

import tofu.*;

public class Memory extends Component {

  private int[] memory = new int[32 * 1024];
  private Node[] dataBus = new Node[8];
  private Node[] addressBus = new Node[16];
  private Node select;
  private Node load;
  private int address;
  private boolean advancePrinter;
  
  public void init() {
    select = getNode("select");
    load = getNode("load");
    for(int i = 0; i < addressBus.length; i++) {
      addressBus[i] = getNode("a" + i);
    }
    for(int i = 0; i < dataBus.length; i++) {
      dataBus[i] = getNode("d" + i);
    }
  }
  
  private int getData() {
    int data = 0;
    for(int i = 7; i >= 0; i--) {
      data <<= 1;
      data |= dataBus[i].getVoltage() == Voltage.H ? 1 : 0;
    }
    return data;
  } 
  
  public void update() {
    address = 0;
    for(int i = 15; i >= 0; i--) {
      address <<= 1;
      address |= addressBus[i].getVoltage() == Voltage.H ? 1 : 0;
    }
    if (load.getVoltage() == Voltage.H) {
      int data = getData();
      if (address == 0xFFFE) {
        advancePrinter = true;
      } else if (advancePrinter && address == 0xFFFF) {
        advancePrinter = false;
        System.out.print((char)data);
      } else {
        memory[0x7FFF & address] = getData();  
      }
    }
  }
  
  public void flow() {
    if (select.getVoltage() == Voltage.H) {
      int data = memory[0x7FFF & address];
      for(int i = 0; i <= 7; i++) {
        if ((data & 0x01) == 1) {
          energizeNode(dataBus[i]);  
        } else if (dataBus[i].getVoltage() != Voltage.Z) {
          System.out.println("Data Bus not prepared for memory read.");
          System.exit(0);
        }
        data >>= 1;
      }  
    }
  }
}
