/*
 * TOFU Circuit Simulator
 * Copyright (C) 2007 Michael Birken
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 * 
 * http://www.meatfighter.com/tofu
 */

package tofu.harryporter;

import java.io.*;
import tofu.*;

public class TapeReader extends Component {

  String tapeFileName;
  private Node[] dataBus = new Node[8];
  private Node select;
  private Node advance;
  private Node end;
  private InputStream in;
  private int data;
  private Voltage lastAdvance;
  
  public void init() {
    select = getNode("select");
    advance = getNode("advance");
    end = getNode("end");
    for(int i = 0; i < dataBus.length; i++) {
      dataBus[i] = getNode("d" + i);
    }
    
    try {
      in = new BufferedInputStream(new FileInputStream(getInitParam()));     
    } catch(IOException e) {
      System.out.println(e.getMessage());
      System.exit(0);
    }
  }
  
  public void update() {
    if (in != null && advance.getVoltage() == Voltage.H 
        && lastAdvance == Voltage.Z) {
      try {        
        data = in.read();
        if (data < 0) {
          in.close();
          in = null;
        }
      } catch(IOException e) {
        System.out.println(e.getMessage());
        System.exit(0);
      }
    }
    lastAdvance = advance.getVoltage();
  }
  
  public void flow() {
    if (select.getVoltage() == Voltage.H) {
      if (this.data < 0) {
        energizeNode(end);
      } else if (advance.getVoltage() == Voltage.Z) {
        int data = this.data;
        for(int i = 0; i <= 7; i++) {
          if ((data & 0x01) == 1) {
            energizeNode(dataBus[i]);  
          }
          data >>= 1;
        } 
      }
    }
  }  
}
